"use strict";
/**
 * @file index.spec.js
 * @author Rafael Kallis <rk@rafaelkallis.com>
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
}
Object.defineProperty(exports, "__esModule", { value: true });
// tslint:disable: variable-name
const Joi = __importStar(require("joi"));
const index_1 = require("../index");
describe('joi schema', () => {
    describe('validateParams', () => {
        it('no arguments, no schemata', () => {
            const f = jest.fn();
            const f_val = index_1.validateParams()(f);
            expect(f_val()).toBeUndefined();
            expect(f).toHaveBeenCalledWith();
        });
        it('argument, no schemata', () => {
            const f = jest.fn();
            const f_val = index_1.validateParams()(f);
            // @ts-ignore: Expected 0 arguments, but got 1.
            expect(() => {
                f_val('a');
            }).toThrow();
            expect(f).not.toHaveBeenCalled();
        });
        it('arguments, no schemata', () => {
            const f = jest.fn();
            const f_val = index_1.validateParams()(f);
            // @ts-ignore: Expected 0 arguments, but got 3.
            expect(() => {
                f_val('a', 'b', 'c');
            }).toThrow();
            expect(f).not.toHaveBeenCalled();
        });
        it('no arguments, schema', () => {
            const f = jest.fn();
            const f_val = index_1.validateParams(Joi.string()
                .required()
                .valid('a'))(f);
            // @ts-ignore: Expected 1 arguments, but got 0.
            expect(() => {
                f_val();
            }).toThrow();
            expect(f).not.toHaveBeenCalled();
        });
        it('no arguments, schemata', () => {
            const f = jest.fn();
            const f_val = index_1.validateParams(Joi.string()
                .required()
                .valid('a'), Joi.string()
                .required()
                .valid('b'))(f);
            // @ts-ignore: Expected 2 arguments, but got 0.
            expect(() => {
                f_val();
            }).toThrow();
            expect(f).not.toHaveBeenCalled();
        });
        it('#arguments !== #schemata', () => {
            const f = jest.fn();
            const f_val = index_1.validateParams(Joi.string()
                .required()
                .valid('a'), Joi.string()
                .required()
                .valid('b'))(f);
            // @ts-ignore: Expected 2 arguments, but got 1.
            expect(() => {
                f_val('a');
            }).toThrow();
            // @ts-ignore: Expected 2 arguments, but got 1.
            expect(() => {
                f_val('b');
            }).toThrow();
            expect(f).not.toHaveBeenCalled();
        });
        it('valid arguments', () => {
            const f = jest.fn();
            const f_val = index_1.validateParams(Joi.string()
                .required()
                .valid('a'), Joi.string()
                .required()
                .valid('b'))(f);
            expect(f_val('a', 'b')).toBeUndefined();
        });
        it('valid arguments optional', () => {
            const f = jest.fn();
            const f_val = index_1.validateParams(Joi.string().optional())(f);
            // @ts-ignore: Expected 1 arguments, but got 0.
            expect(f_val()).toBeUndefined();
            expect(f).toHaveBeenCalledWith(undefined);
        });
        it('invalid arguments', () => {
            const f = jest.fn();
            const f_val = index_1.validateParams(Joi.string()
                .required()
                .valid('a'), Joi.string()
                .required()
                .valid('b'))(f);
            expect(() => {
                f_val('b', 'c');
            }).toThrow();
        });
        it('passthrough arguments', () => {
            const f = jest.fn();
            const f_val = index_1.validateParams(Joi.string()
                .required()
                .valid('a'), Joi.string()
                .required()
                .valid('b'))(f);
            f_val('a', 'b');
            expect(f).toHaveBeenCalledWith('a', 'b');
        });
        it('return function result', () => {
            const f = jest.fn(() => 'result');
            const f_val = index_1.validateParams()(f);
            expect(f_val()).toBe('result');
            expect(f).toHaveBeenCalledWith();
        });
        it('should bind context', () => {
            let y;
            const x = {
                f: index_1.validateParams(Joi.string()
                    .required()
                    .valid('a'))(jest.fn(function () {
                    y = this;
                })),
            };
            expect(x.f('a')).toBeUndefined();
            expect(x).toEqual(y);
        });
    });
    describe('validateResult', () => {
        it('valid funtion result', () => {
            const f = jest.fn(() => 'result');
            const f_val = index_1.validateResult(Joi.string().valid('result'))(f);
            expect(f_val()).toBe('result');
            expect(f).toHaveBeenCalledWith();
        });
        it('invalid funtion result', () => {
            const f = jest.fn(() => 'result');
            const f_val = index_1.validateResult(Joi.string().valid('wrong'))(f);
            expect(() => {
                f_val();
            }).toThrow();
            expect(f).toHaveBeenCalledWith();
        });
        it('pass arguments', () => {
            const f = jest.fn(() => 'result');
            const f_val = index_1.validateResult(Joi.string())(f);
            // @ts-ignore: Expected 0 arguments, but got 2.
            expect(f_val('a', 'b')).toBe('result');
            expect(f).toHaveBeenCalledWith('a', 'b');
        });
        it('promise result', () => __awaiter(this, void 0, void 0, function* () {
            const f = jest.fn(() => __awaiter(this, void 0, void 0, function* () { return 'result'; }));
            const f_val = index_1.validateResult(Joi.string())(f);
            expect(yield f_val()).toBe('result');
        }));
        it('promise rejection', () => __awaiter(this, void 0, void 0, function* () {
            const f = jest.fn(() => __awaiter(this, void 0, void 0, function* () { return 'wrong'; }));
            const f_val = index_1.validateResult(Joi.string().valid('result'))(f);
            expect(f_val()).rejects.toBeDefined();
        }));
        it('should bind context', () => {
            let y;
            const x = {
                f: index_1.validateResult(Joi.string()
                    .required()
                    .valid('result'))(jest.fn(function () {
                    y = this;
                    return 'result';
                })),
            };
            expect(x.f()).toBe('result');
            expect(x).toEqual(y);
        });
    });
});
