"use strict";
/**
 * @file index.ts
 * @author Rafael Kallis <rk@rafaelkallis.com>
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
}
Object.defineProperty(exports, "__esModule", { value: true });
const invariant_1 = __importDefault(require("invariant"));
const is_promise_1 = __importDefault(require("is-promise"));
const lodash_isfunction_1 = __importDefault(require("lodash.isfunction"));
/**
 * Create a parameter validation function.
 * @param validate - The function which validates a value given a schema.
 */
function parameterValidationFactory(validate) {
    return function (...schemata) {
        return function (f) {
            invariant_1.default(lodash_isfunction_1.default(f), 'argument f is not a function');
            return function () {
                invariant_1.default(schemata.length >= arguments.length, 'number of arguments is greater than number of schemata.');
                const newArgs = [];
                for (let i = 0; i < schemata.length; i++) {
                    newArgs[i] = validate(schemata[i], arguments[i]);
                }
                return f.apply(this, newArgs);
            };
        };
    };
}
/**
 * Create a result validation function.
 * @param validate - The function which validates a value given a schema.
 */
function resultValidationFactory(validate) {
    return function (s) {
        invariant_1.default(s, 'argument s is not a schema');
        return function (f) {
            invariant_1.default(lodash_isfunction_1.default(f), 'argument f is not a function');
            return function () {
                const result = f.apply(this, arguments);
                if (is_promise_1.default(result)) {
                    return result.then(v => validate(s, v));
                }
                return validate(s, result);
            };
        };
    };
}
/**
 * Joi schema validator.
 * @param schema - Joi schema.
 * @param arg - Argument to validate.
 * @return The validated argument.
 */
function joiValidation(schema, arg) {
    const { value, error } = schema.validate(arg);
    // tslint:disable-next-line: strict-type-predicates
    if (error !== null) {
        throw error;
    }
    return value;
}
// Parameter Validation
const joiParameterValidation = parameterValidationFactory(joiValidation);
/**
 * Returns parameter validating higher order function.
 * @param schemata - Joi schemata for parameters.
 * @returns Parameter validating higher order function.
 */
function validateParams(...schemata) {
    for (const schema of schemata) {
        invariant_1.default(schema.isJoi, 'provided schema is not a Joi schema');
    }
    return joiParameterValidation(...schemata);
}
exports.validateParams = validateParams;
// Result Validation
const joiResultValidation = resultValidationFactory(joiValidation);
/**
 * Returns result validating higher order function.
 * @param schema - Joi schema for result.
 * @returns Result validating higher order function.
 */
function validateResult(schema) {
    invariant_1.default(schema.isJoi, 'provided schema is not a Joi schema');
    return joiResultValidation(schema);
}
exports.validateResult = validateResult;
